@@ -87,6 +87,7 @@ def train_from_folder(
     attn_res_layers = [32],
     sle_spatial = False,
     disc_output_size = 1,
+    antialias = False,
     interpolation_num_steps = 100,
     save_frames = False,
     num_image_tiles = 8,
@@ -108,6 +109,7 @@ def train_from_folder(
         attn_res_layers = cast_list(attn_res_layers),
         sle_spatial = sle_spatial,
         disc_output_size = disc_output_size,
+        antialias = antialias,
         image_size = image_size,
         optimizer = optimizer,
         fmap_max = fmap_max,
